//0. REQUIRED LIBRARIES
#include <Servo.h>
#include <IRremote.h>

//1. NAMING MOTORS
// Motor A
const int motorRE = 3;
const int motorRF = 2;
const int motorRB = 4;
// Motor B
const int motorLE = 5;
const int motorLF = 7;
const int motorLB = 8;

//2. NAMING ULTRASONIC SENSOR
const int trigPin = 12;
const int echoPin = 13;
//2.1 ULTRASONIC SENSOR VARIABLES
long duration;
int distance;
int randNumber;

//3. NAMING IR RECEIVER
const int receiver = 11; // Signal Pin of IR receiver to Arduino Digital Pin 11
//3.1 DECLARE OBJECTS
IRrecv irrecv(receiver);     // create instance of 'irrecv'
decode_results results;      // create instance of 'decode_results'
//3.2 INTEGER
int irActivated;

//4. NAMING JOYSTICK INPUTS
const int X_pin = A0; // analog pin connected to X output
const int Y_pin = A1; // analog pin connected to Y output
//4.1 JOYSTICK VARIABLES
int xRead; int yRead;

//5. SETTING UP SERVO
Servo myservo;  // create servo object to control a servo
const int servoPin = 9;
int startPos = 90;
int pos;    // variable to store the servo position

void setup() {
  // 0. INITIATE SERIAL, CAN BE ERASED AFTER TRIALS
  Serial.begin(9600);

  // 1. MOTORS
  pinMode(motorRE, OUTPUT);
  pinMode(motorLE, OUTPUT);
  pinMode(motorRF, OUTPUT);
  pinMode(motorRB, OUTPUT);
  pinMode(motorLF, OUTPUT);
  pinMode(motorLB, OUTPUT);

  //2. ULTRASONIC SENSOR
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);

  //3. IR RECEIVER
  Serial.println("IR Receiver Button Decode");
  irrecv.enableIRIn(); // Start the receiver

  //4. JOYSTICK
  pinMode(X_pin, INPUT);
  pinMode(Y_pin, INPUT);

  //5. SERVO
  myservo.attach(servoPin, 1000, 2000);  // attaches the servo on pin 9 to the servo object
}

void loop() { 
  //1.1 PRIORITY A - Check Joystick
  //1.0 Check if joystick has been activated
  //If yes - activate evasion action
  xRead = analogRead(X_pin);
  yRead = analogRead(Y_pin);

  //if joystick is moved forward move servo right
  if (xRead < 300 || xRead > 700) {
    for (pos = startPos; pos < 170; pos += 5) {
      myservo.write(pos);
      delay(100);}
    for (pos = 170; pos < startPos; pos -= 5){
      myservo.write(pos);
      delay(200);}
  }
  
//if joystick is moved sideways move servo left
  if (yRead < 300 || yRead > 700) {
      for (pos = startPos; pos < 0; pos -= 5) {
        myservo.write(pos);
        delay(100); }
      for (pos = 0; pos < startPos; pos += 5){
        myservo.write(pos);
        delay(200);}
  }
  
  //2.0 PRIORITY B - Check Ultrasound and IR receiver
  //2.1 Run ultrasound
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);

  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);

  duration = pulseIn(echoPin, HIGH);
  distance = duration * 0.034 / 2;
  //If less than 50cm evade
     if (distance < 50) {
      back();
      delay(500);
     //turn left or right randomly after going back
      randNumber = random(2);
      if (randNumber == 0 ) {
        left();
        delay(500); }
      else {
        right();
        delay(500);
        }
      }  
  else {
    forward();
  }

  //2.2 Check IR Sensor
  if (results.value == 551508645 || results.value == 551512725 || results.value == 551545365) { //Check if conroller is clicking 'OK', 'UP' or 'DOWN'
    irActivated = true;
    }
    else {
      irActivated = false;
    }
  
  //2.3 Activate evasion
  if (irActivated) {
    if (results.value == 551508645){//check for ok
      forward(); 
      }
     }
    else {
      stopMotion();
  }
}


//Functions
void forward() {
  digitalWrite(motorRF, HIGH);
  digitalWrite(motorRB, LOW);
  analogWrite(motorRE, 150); //speed between 0 and 255

  digitalWrite(motorLF, HIGH);
  digitalWrite(motorLB, LOW);
  analogWrite(motorLE, 150);
}

void back() {
  digitalWrite(motorRF, LOW);
  digitalWrite(motorRB, HIGH);
  analogWrite(motorRE, 100);

  digitalWrite(motorLF, LOW);
  digitalWrite(motorLB, HIGH);
  analogWrite(motorLE, 100);
}

void right() {
  digitalWrite(motorRF, LOW);
  digitalWrite(motorRB, HIGH);
  analogWrite(motorRE, 100);

  digitalWrite(motorLF, HIGH);
  digitalWrite(motorLB, LOW);
  analogWrite(motorLE, 100);
}

void left() {
  digitalWrite(motorRF, HIGH);
  digitalWrite(motorRB, LOW);
  analogWrite(motorRE, 100);

  digitalWrite(motorLF, LOW);
  digitalWrite(motorLB, HIGH);
  analogWrite(motorLE, 100);
}

void stopMotion() {
  digitalWrite(motorRF, LOW);
  digitalWrite(motorRB, LOW);

  digitalWrite(motorLF, LOW);
  digitalWrite(motorLB, LOW);
}
